package auth

import (
	"fmt"

	"github.com/pivotal-cf/on-demand-service-broker/boshclient"
	"github.com/pivotal-cf/on-demand-service-broker/mockbosh"

	. "github.com/onsi/ginkgo"
	. "github.com/onsi/gomega"
)

var _ = Describe("BearerAuth Header Builder", func() {
	var (
		mockUAA              *mockbosh.UAA
		authorizer           boshclient.AuthHeaderBuilder
		actualClientID       = "some-username"
		actualClientSecret   = "some-password"
		suppliedClientID     string
		suppliedClientSecret string
		tokenToReturn        = "some-token"
		buildErr             error
		header               string
	)

	BeforeEach(func() {
		mockUAA = mockbosh.NewUAA(actualClientID, actualClientSecret, tokenToReturn)
	})

	AfterEach(func() {
		mockUAA.Close()
	})

	JustBeforeEach(func() {
		authorizer = NewBearerAuthHeaderBuilder(mockUAA.URL, suppliedClientID, suppliedClientSecret)
		header, buildErr = authorizer.Build()
	})

	Context("valid clientID and secret", func() {
		BeforeEach(func() {
			suppliedClientID = actualClientID
			suppliedClientSecret = actualClientSecret
		})

		It("succeeds", func() {
			Expect(buildErr).NotTo(HaveOccurred())
		})

		It("gets a token from UAA", func() {
			Expect(header).To(Equal(fmt.Sprintf("Bearer %s", tokenToReturn)))
		})
	})

	Context("invalid clientID and secret", func() {
		BeforeEach(func() {
			suppliedClientID = "wrong"
			suppliedClientSecret = "wronger"
		})

		It("fails", func() {
			Expect(buildErr).To(MatchError(ContainSubstring(mockbosh.UnauthorizedError)))
			Expect(buildErr).To(MatchError(ContainSubstring(mockbosh.UnauthorizedErrorDescription)))
		})
	})

	Context("malformed response", func() {
		BeforeEach(func() {
			suppliedClientID = mockbosh.MalformedResponseUser
			suppliedClientSecret = ""
		})

		It("fails", func() {
			Expect(buildErr).To(MatchError(ContainSubstring("no access token in grant")))
		})
	})

	Context("uaa 500", func() {
		BeforeEach(func() {
			suppliedClientID = mockbosh.InternalServerErrorUser
			suppliedClientSecret = ""
		})

		It("fails", func() {
			Expect(buildErr).To(MatchError(ContainSubstring(mockbosh.InternalServerErrorMessage)))
		})
	})
})
