package boshclient

import (
	"bytes"
	"fmt"
	"net/http"

	"gopkg.in/yaml.v2"
)

func (c *Client) Deploy(manifest BoshManifest) (int, error) {
	manifestBytes, err := yaml.Marshal(manifest)
	if err != nil {
		return 0, fmt.Errorf("error marshalling manifest to yaml: %s", err)
	}
	return c.DeployRaw(manifestBytes)
}

func (c *Client) DeployRaw(manifest []byte) (int, error) {
	req, err := http.NewRequest("POST", fmt.Sprintf("%s/deployments", c.boshURL), bytes.NewReader(manifest))
	if err != nil {
		return 0, err
	}

	authHeader, err := c.authHeaderBuilder.Build()
	if err != nil {
		return 0, err
	}

	req.Header.Set("Authorization", authHeader)

	req.Header.Set("Content-Type", "text/yaml")
	response, err := c.HTTPClient.Do(req)
	if err != nil {
		return 0, fmt.Errorf("error contacting BOSH director: %s", err)
	}

	if response.StatusCode != http.StatusFound {
		return 0, unexpectedStatusErr(response, http.StatusFound)
	}

	return taskIDFromRedirect(response)
}
