package boshclient

import (
	"encoding/json"
	"fmt"
	"net/http"

	"gopkg.in/yaml.v2"
)

func (c *Client) GetManifestRaw(name string) ([]byte, error) {
	req, err := http.NewRequest("GET", fmt.Sprintf("%s/deployments/%s", c.boshURL, name), nil)
	if err != nil {
		return nil, err
	}

	authHeader, err := c.authHeaderBuilder.Build()
	if err != nil {
		return nil, err
	}

	req.Header.Set("Authorization", authHeader)

	resp, err := c.HTTPClient.Do(req)
	if err != nil {
		return nil, err
	}
	defer resp.Body.Close()

	if resp.StatusCode == http.StatusNotFound {
		return nil, nil
	}

	if resp.StatusCode != http.StatusOK {
		return nil, unexpectedStatusErr(resp, http.StatusOK)
	}

	respJSON := make(map[string]string)
	if err := json.NewDecoder(resp.Body).Decode(&respJSON); err != nil {
		return nil, fmt.Errorf("error unmarshalling GetManifest json response: %s", err)
	}

	return []byte(respJSON["manifest"]), nil
}

func (c *Client) GetManifest(name string) (*BoshManifest, error) {
	data, err := c.GetManifestRaw(name)
	if err != nil {
		return nil, err
	}
	if data == nil {
		return nil, nil
	}

	var manifest BoshManifest
	if err := yaml.Unmarshal(data, &manifest); err != nil {
		return nil, fmt.Errorf("error unmarshalling manifest yaml from %s: %s", data, err)
	}

	return &manifest, nil
}
