package boshclient

import (
	"encoding/json"
	"fmt"
	"net/http"
)

func (c *Client) GetTaskState(taskID int) (BoshTask, error) {
	errs := func(err error) (BoshTask, error) {
		return BoshTask{}, err
	}

	req, err := http.NewRequest("GET", fmt.Sprintf("%s/tasks/%d", c.boshURL, taskID), nil)
	if err != nil {
		return errs(err)
	}

	authHeader, err := c.authHeaderBuilder.Build()
	if err != nil {
		return errs(err)
	}

	req.Header.Set("Authorization", authHeader)

	response, err := c.HTTPClient.Do(req)
	if err != nil {
		return errs(fmt.Errorf("error reaching bosh director: %s", err))
	}

	if response.StatusCode != http.StatusOK {
		return errs(fmt.Errorf("expected status 200, got %d", response.StatusCode))
	}

	var getTaskResponse BoshTask
	defer response.Body.Close()
	if err := json.NewDecoder(response.Body).Decode(&getTaskResponse); err != nil {
		return errs(fmt.Errorf("error unmarshalling get_task response from bosh director: %s", err))
	}

	return getTaskResponse, nil
}

type TaskOutput struct {
	ExitCode int    `json:"exit_code"`
	StdOut   string `json:"stdout"`
	StdErr   string `json:"stderr"`
}

func (c *Client) GetTaskOutput(taskID int) ([]TaskOutput, error) {
	resp, err := c.getOutput(taskID)
	if err != nil {
		return nil, err
	}
	defer resp.Close()

	outputs := []TaskOutput{}

	decoder := json.NewDecoder(resp)
	for decoder.More() {
		var output TaskOutput
		if err := decoder.Decode(&output); err != nil {
			return nil, fmt.Errorf("error decoding bosh output response: %s", err)
		}
		outputs = append(outputs, output)
	}

	return outputs, nil
}
