package boshclient_test

import (
	"github.com/pivotal-cf/on-demand-service-broker/boshclient"

	. "github.com/onsi/ginkgo"
	. "github.com/onsi/gomega"
)

var _ = Describe("tasks", func() {
	var taskID = 2112
	Context("getting task state", func() {
		var (
			taskState  boshclient.BoshTask
			getTaskErr error
		)

		JustBeforeEach(func() {
			director.InitializeTaskTest(taskID, boshclient.BoshTask{State: "a-state", Description: "a-description"}, shouldFail)
			taskState, getTaskErr = c.GetTaskState(taskID)
		})

		Context("when bosh fetches the task successfully", func() {
			BeforeEach(func() {
				shouldFail = false
			})

			It("returns the task state", func() {
				Expect(taskState).To(Equal(boshclient.BoshTask{State: "a-state", Description: "a-description"}))
			})

			It("does not error", func() {
				Expect(getTaskErr).NotTo(HaveOccurred())
			})
		})

		Context("when bosh fails to fetch the task", func() {
			BeforeEach(func() {
				shouldFail = true
			})

			It("wraps the error", func() {
				Expect(getTaskErr).To(MatchError("expected status 200, got 500"))
			})
		})
	})
	Context("getting task output", func() {
		var (
			actualTaskOutput    []boshclient.TaskOutput
			actualTaskOutputErr error
			expectedTaskOutputs []boshclient.TaskOutput
		)
		JustBeforeEach(func() {
			director.InitializeTaskOutputTest(taskID, expectedTaskOutputs, shouldFail)
			actualTaskOutput, actualTaskOutputErr = c.GetTaskOutput(taskID)
		})

		Context("when the client fetches task output of a valid errand task", func() {
			BeforeEach(func() {
				expectedTaskOutputs = []boshclient.TaskOutput{
					{ExitCode: 1, StdOut: "a-description", StdErr: "err"},
					{ExitCode: 2, StdOut: "a-description", StdErr: "err"},
					{ExitCode: 3, StdOut: "a-description", StdErr: "err"},
					{ExitCode: 5, StdOut: "a-description", StdErr: "err"},
				}
				shouldFail = false
			})
			It("returns the correct output of the task", func() {
				Expect(actualTaskOutputErr).NotTo(HaveOccurred())
				Expect(actualTaskOutput).To(ConsistOf(expectedTaskOutputs))
			})
		})

		Context("when there is no output", func() {
			BeforeEach(func() {
				expectedTaskOutputs = []boshclient.TaskOutput{}
				shouldFail = false
			})
			It("returns the correct output of the task", func() {
				Expect(actualTaskOutputErr).NotTo(HaveOccurred())
				Expect(actualTaskOutput).To(BeEmpty())
			})
		})
		Context("when bosh fails to fetch the output", func() {
			BeforeEach(func() {
				shouldFail = true
			})

			It("returns an error", func() {
				Expect(actualTaskOutputErr).To(MatchError(ContainSubstring("expected status 200, was 500.")))
			})
		})
	})
})
