package boshclient

import (
	"bytes"
	"encoding/json"
	"fmt"
	"net/http"
)

func (c *Client) RunErrand(deploymentName, errandName string) (int, error) {
	errandBodyBytes, err := json.Marshal(ErrandRequest{KeepAlive: false})
	if err != nil {
		return 0, fmt.Errorf("error marshalling errand request to yaml: %s", err)
	}
	req, err := http.NewRequest("POST", fmt.Sprintf("%s/deployments/%s/errands/%s/runs", c.boshURL, deploymentName, errandName), bytes.NewReader(errandBodyBytes))
	if err != nil {
		return 0, err
	}

	authHeader, err := c.authHeaderBuilder.Build()
	if err != nil {
		return 0, err
	}

	req.Header.Set("Authorization", authHeader)

	req.Header.Add("Content-Type", "application/json")
	response, err := c.HTTPClient.Do(req)
	if err != nil {
		return 0, fmt.Errorf("error contacting BOSH director: %s", err)
	}

	if response.StatusCode != http.StatusFound {
		return 0, unexpectedStatusErr(response, http.StatusFound)
	}

	return taskIDFromRedirect(response)
}

type ErrandRequest struct {
	KeepAlive bool `json:"keep-alive"`
}
