package boshclient_test

import (
	. "github.com/onsi/ginkgo"
	. "github.com/onsi/gomega"
	"github.com/pivotal-cf/on-demand-service-broker/boshclient"
	"github.com/pivotal-cf/on-demand-service-broker/mockbosh"
)

var _ = Describe("running errands", func() {
	taskIDToReturn := 5
	deploymentName := "deploymentName"
	errandName := "errandName"

	var actualErr error
	BeforeEach(func() {
		director.InitializeVMsTest(taskIDToReturn, false)
	})
	JustBeforeEach(func() {
		_, actualErr = c.RunErrand(deploymentName, errandName)
	})
	Context("succesfully", func() {
		BeforeEach(func() {
			director.InitializeTaskTest(taskIDToReturn, boshclient.BoshTask{State: boshclient.Done}, false)
		})

		It("invokes BOSH to queue up an errand", func() {
			Expect(director.ErrandCalls).To(Equal(1))
			Expect(director.LastErrandRun).To(Equal(mockbosh.Errand{Name: errandName, DeploymentName: deploymentName}))
			Expect(actualErr).NotTo(HaveOccurred())
		})
	})
	Context("has an error", func() {
		BeforeEach(func() {
			director.InitializeTaskTest(taskIDToReturn, boshclient.BoshTask{State: boshclient.Done}, true)
		})

		It("invokes BOSH to queue up an errand", func() {
			Expect(director.ErrandCalls).To(Equal(1))
			Expect(director.LastErrandRun).To(Equal(mockbosh.Errand{Name: errandName, DeploymentName: deploymentName}))
			Expect(actualErr).To(HaveOccurred())
		})
	})
})
