package broker

import "github.com/pivotal-cf/brokerapi"

func (b *Broker) Services() []brokerapi.Service {
	servicePlans := []brokerapi.ServicePlan{}
	for _, plan := range b.serviceOffering.Plans {
		servicePlan := brokerapi.ServicePlan{
			ID:          plan.ID,
			Name:        plan.Name,
			Description: plan.Description,
			Metadata: &brokerapi.ServicePlanMetadata{
				DisplayName: plan.Metadata.DisplayName,
				Bullets:     plan.Metadata.Bullets,
			},
		}
		servicePlans = append(servicePlans, servicePlan)
	}

	return []brokerapi.Service{
		{
			ID:            b.serviceOffering.ID,
			Name:          b.serviceOffering.Name,
			Description:   b.serviceOffering.Description,
			Bindable:      b.serviceOffering.Bindable,
			PlanUpdatable: b.serviceOffering.PlanUpdatable,
			Plans:         servicePlans,
			Metadata: &brokerapi.ServiceMetadata{
				DisplayName: b.serviceOffering.Metadata.DisplayName,
			},
			Requires: requiredPermissions(b.serviceOffering.Requires),
			Tags:     b.serviceOffering.Tags,
		},
	}
}

func requiredPermissions(permissions []string) []brokerapi.RequiredPermission {
	brokerPermissions := []brokerapi.RequiredPermission{}
	for _, permission := range permissions {
		brokerPermissions = append(brokerPermissions, brokerapi.RequiredPermission(permission))
	}
	return brokerPermissions
}
