package broker

import (
	"fmt"

	"github.com/pivotal-cf/brokerapi"
)

func (b *Broker) Deprovision(instanceID string, _ brokerapi.DeprovisionDetails, asyncAllowed bool) (brokerapi.IsAsync, error) {
	if !asyncAllowed {
		return true, brokerapi.ErrAsyncRequired
	}
	exists, err := b.instanceRepo.InstanceExists(instanceID)
	if err != nil {
		wrappedError := fmt.Errorf("error deprovisioning: checking instance existence: %s", err)
		b.logger.Println(wrappedError)
		return true, wrappedError
	}

	if !exists {
		b.logger.Println(fmt.Errorf("error deprovisioning: instance %s, not found ", instanceID))
		return true, brokerapi.ErrInstanceDoesNotExist
	}

	instance, err := b.instanceRepo.FindByID(instanceID)
	if err != nil {
		wrappedError := fmt.Errorf("error deprovisioning: finding instance: %s", err)
		b.logger.Println(wrappedError)
		return true, wrappedError
	}

	b.logger.Printf("deleting deployment for instance %s\n", instanceID)
	taskID, err := b.boshClient.DeleteDeployment(b.deploymentName(instanceID))
	if err != nil {
		wrappedError := fmt.Errorf("error deprovisioning: deleting bosh deployment: %s", err)
		b.logger.Println(wrappedError)
		return true, wrappedError
	}

	b.logger.Printf("Bosh task id for Delete instance %s was %d\n", instanceID, taskID)

	if err := b.instanceRepo.Save(Instance{ID: instanceID, BoshTaskID: &taskID, LastOperationType: Delete, PlanID: instance.PlanID}); err != nil {
		wrappedError := fmt.Errorf("error deprovisioning: writing task ID: %s", err)
		b.logger.Println(wrappedError)
		return true, wrappedError
	}

	return true, nil
}
