package broker

import "fmt"

func (b *Broker) redeploy(instance Instance, planID string, lastOperation OperationType, arbitraryParams map[string]interface{}) (int, error) {
	if instance.BoshTaskID != nil {
		return 0, SkippableError{fmt.Errorf("deployment is still create-in-progress")}
	}

	deploymentName := b.deploymentName(instance.ID)
	oldManifest, err := b.boshClient.GetManifestRaw(deploymentName)
	if err != nil {
		return 0, SkippableError{fmt.Errorf("error getting old manifest: %s", err)}
	}
	if oldManifest == nil {
		return 0, SkippableError{fmt.Errorf("there is no old manifest, the instance must not exist")}
	}
	instance, err = b.deployer.Deploy(instance, planID, deploymentName, lastOperation, arbitraryParams, oldManifest)
	if err != nil {
		return 0, err
	}

	return *instance.BoshTaskID, nil
}

const InstancePrefix = "service-instance_"

func (b *Broker) deploymentName(instanceId string) string {
	return InstancePrefix + instanceId
}
