package broker

import "github.com/pivotal-cf/on-demand-service-broker/serviceadapter"

type ServiceCatalog struct {
	ID            string
	Name          string `yaml:"service_name"`
	Description   string `yaml:"service_description"`
	Bindable      bool
	PlanUpdatable bool     `yaml:"plan_updatable"`
	Requires      []string `yaml:"requires,omitempty"`
	Metadata      Metadata // same type as for plans, even though a service has no bullets
	Tags          []string
	Plans         []Plan
}

type Plan struct {
	ID             string `yaml:"plan_id"`
	Name           string
	Description    string
	Metadata       Metadata
	Quotas         *Quotas `yaml:"quotas,omitempty"`
	Properties     serviceadapter.Properties
	InstanceGroups []serviceadapter.InstanceGroup `yaml:"instance_groups,omitempty"`
}

type Metadata struct {
	DisplayName string   `yaml:"display_name"`
	Bullets     []string `yaml:"bullets,omitempty"`
}

type Quotas struct {
	ServiceInstanceLimit int `yaml:"service_instance_limit"`
}

func (c ServiceCatalog) FindPlanById(planId string) *Plan {
	for _, plan := range c.Plans {
		if plan.ID == planId {
			return &plan
		}
	}
	return nil
}
